@@ -9,7 +9,7 @@ from collections import namedtuple
 import pickle
 import types as build_in_types
 
-__version__ = "0.9.6"
+__version__ = "0.9.9"
 
 class EventCollector:
     _submit_collector = {}
@@ -50,6 +50,7 @@ class TelebotForms:
     GLOBAL_STOP_FREEZE_TEXT = None
     GLOBAL_LIFE_TIME = None
     GLOBAL_INVALID_INPUT_TEXT = None
+    GLOBAL_FORM_IMG = None
 
 
     def __init__(self,bot,fsm=None):
@@ -78,8 +79,9 @@ class TelebotForms:
 
             form = BaseForm.form_loads(fsm_data.args.form)
             settings = self._get_form_settings(form,prepare_update=user_id)
-            if fsm_data.args.from_callback and settings["FREEZE_MODE"]:
-                return True
+            if hasattr(fsm_data.args, 'from_callback'):
+                if fsm_data.args.from_callback and settings["FREEZE_MODE"]:
+                    return True
             return False
 
 
@@ -237,6 +239,11 @@ class TelebotForms:
             settings["INVALID_INPUT_TEXT"] = self.GLOBAL_INVALID_INPUT_TEXT
         else:
             settings["INVALID_INPUT_TEXT"] = form.default_input_not_valid
+        if form.FORM_IMG:
+            settings["FORM_IMG"] = form.FORM_IMG
+        else:
+            settings["FORM_IMG"] = self.GLOBAL_FORM_IMG
+
         if prepare_update:
             for prepere_keyname in prepere_list:
                 if isinstance(settings[prepere_keyname], build_in_types.FunctionType):
@@ -275,7 +282,10 @@ class TelebotForms:
         parse_mode = None
         if form.form_title:
             parse_mode = "Markdown"
-        msg = self.bot.send_message(user_id,text,reply_markup=keyboard,parse_mode=parse_mode,disable_web_page_preview=True)
+        if not settings["FORM_IMG"]:
+            msg = self.bot.send_message(user_id,text,reply_markup=keyboard,parse_mode=parse_mode,disable_web_page_preview=True)
+        else:
+            msg = self.bot.send_photo(user_id,settings["FORM_IMG"],caption=text,reply_markup=keyboard,parse_mode=parse_mode)
         form.last_msg_id = msg.message_id
         self.fsm.set_state(int(user_id),idle_state,life_time = settings["LIFE_TIME"],form=form._form_dumps())
         return msg
@@ -302,6 +312,7 @@ class TelebotForms:
         cancel_key = types.InlineKeyboardButton(text=settings["CANCEL_BUTTON_TEXT"], callback_data=settings["CANCEL_CALLBACK"])
         if not field.without_system_key:
             keyboard.add(cancel_key)
+        field.before_input_update(self,form,call)
         self.bot.delete_message(call.message.chat.id,call.message.message_id)
         msg = self.bot.send_message(call.from_user.id,text,reply_markup=keyboard)
         form.last_msg_id = msg.message_id
@@ -361,6 +372,11 @@ class TelebotForms:
         settings = self._get_form_settings(form,prepare_update=message.chat.id)
         f_id = state_data.args.field_id
         field = form.get_field_by_id(f_id)
+
+        if field.value_from_message_manual_mode:
+            field.manualy_handle_message(self,message,form)
+            return
+        field.after_input_update(self,form,message)
         new_value = message.text
         valid = True
         if not field.validate(message):
@@ -380,6 +396,7 @@ class TelebotForms:
                 error_text = form.form_global_error_message
             self.bot.reply_to(message,error_text)
 
+    
     def callback_mode_input(self,call):
         state_data = self.fsm.get_state(int(call.from_user.id))
         form = BaseForm.form_loads(state_data.args.form)
@@ -388,6 +405,7 @@ class TelebotForms:
         if field.value_from_callback_manual_mode:
             field.manualy_handle_callback(self,call,form)
             return
+        field.after_input_update(self,form,call)
         new_value_id = call.data.split(":")[2]
         new_value = field.get_variable_data(new_value_id)
         field.value = field.format_return_value(new_value)
@@ -414,6 +432,7 @@ class BaseForm:
     MISSING_VALUE_ICON = "💢"
     READ_ONLY_ICON = '🔒'
     EDIT_ICON = '✏️'
+    FORM_IMG  = None
     update_name = None
     custom_button = None
 
